/*
  SPDX-FileCopyrightText: 2021-2025 Laurent Montel <montel@kde.org>

  SPDX-License-Identifier: LGPL-2.0-or-later
*/
#include "kcm_kmail.cpp"
#include <KPluginFactory>

K_PLUGIN_CLASS_WITH_JSON(ConfigurePluginPage, "kmail_config_plugins.json")

#include "kcm_kmail_plugins.moc"
