/*
 *   kmail: KDE mail client
 *   SPDX-FileCopyrightText: 2000 Espen Sand <espen@kde.org>
 *   SPDX-FileCopyrightText: 2001-2003 Marc Mutz <mutz@kde.org>
 *   SPDX-FileCopyrightText: 2021-2025 Laurent Montel <montel@kde.org>
 *
 *   SPDX-License-Identifier: GPL-2.0-or-later
 *
 */

#include "configuredialog/configureaccountpage.h"
#include "configuredialog/configureappearancepage.h"
#include "configuredialog/configurecomposerpage.h"
#include "configuredialog/configuremiscpage.h"
#include "configuredialog/configurepluginpage.h"
#include "configuredialog/configuresecuritypage.h"
